/*
 * ioctl.c  -- ioctl procedures
 *
 * $Id: ioctl.c,v 1.3 2003/09/21 17:37:35 hos Exp $
 *
 */


#include "main.h"


NTSTATUS ioctl_get_dev_desc(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_get_dev_attr(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_get_rep_desc(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_read_rep(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_write_rep(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_get_feature(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);
NTSTATUS ioctl_set_feature(IN PDEVICE_OBJECT dev_obj, IN PIRP irp);


struct uint_ptr_pair ioctl_func_list[] = {
    {IOCTL_HID_GET_DEVICE_DESCRIPTOR, ioctl_get_dev_desc},
    {IOCTL_HID_GET_DEVICE_ATTRIBUTES, ioctl_get_dev_attr},
    {IOCTL_HID_GET_REPORT_DESCRIPTOR, ioctl_get_rep_desc},
    {IOCTL_HID_READ_REPORT, ioctl_read_rep},
    {IOCTL_HID_WRITE_REPORT, ioctl_write_rep},

    {0, NULL}
};


NTSTATUS ioctl_get_dev_desc(IN PDEVICE_OBJECT dev_obj, IN PIRP irp)
{
    device_extension_t *dev_ext;
    PIO_STACK_LOCATION irp_sp;
    PHID_DESCRIPTOR hid_desc;

    dev_ext = GET_DEV_EXT(dev_obj);
    irp_sp = IoGetCurrentIrpStackLocation(irp);
    hid_desc = (PHID_DESCRIPTOR)irp->UserBuffer;

    if(irp_sp->Parameters.DeviceIoControl.OutputBufferLength <
       sizeof(*hid_desc)) {
        return STATUS_BUFFER_TOO_SMALL;
    }

    RtlZeroMemory(hid_desc, sizeof(*hid_desc));
    hid_desc->bLength = sizeof(*hid_desc);
    hid_desc->bDescriptorType = HID_HID_DESCRIPTOR_TYPE;
    hid_desc->bcdHID = HID_REVISION; /* ? */
    hid_desc->bCountry = 0;
    hid_desc->bNumDescriptors = 1;
    hid_desc->DescriptorList[0].bReportType = HID_REPORT_DESCRIPTOR_TYPE; /* 0x22? */
    hid_desc->DescriptorList[0].wReportLength = dev_ext->report_size;

    irp->IoStatus.Information = sizeof(*hid_desc);

    return STATUS_SUCCESS;
}

NTSTATUS ioctl_get_dev_attr(IN PDEVICE_OBJECT dev_obj, IN PIRP irp)
{
    device_extension_t *dev_ext;
    PIO_STACK_LOCATION irp_sp;
    PHID_DEVICE_ATTRIBUTES hid_attr;
    
    dev_ext = GET_DEV_EXT(dev_obj);
    irp_sp = IoGetCurrentIrpStackLocation(irp);
    hid_attr = (PHID_DEVICE_ATTRIBUTES)irp->UserBuffer;

    if(irp_sp->Parameters.DeviceIoControl.OutputBufferLength <
       sizeof(*hid_attr)) {
        return STATUS_BUFFER_TOO_SMALL;
    }

    hid_attr->Size = sizeof(*hid_attr);
    hid_attr->VendorID = dev_ext->vendor_id;
    hid_attr->ProductID = dev_ext->product_id;
    hid_attr->VersionNumber = 1;

    irp->IoStatus.Information = sizeof(*hid_attr);

    return STATUS_SUCCESS;
}

NTSTATUS ioctl_get_rep_desc(IN PDEVICE_OBJECT dev_obj, IN PIRP irp)
{
    PIO_STACK_LOCATION irp_sp;
    ULONG size;
    BYTE *buf;
    NTSTATUS nt_ret;

    irp_sp = IoGetCurrentIrpStackLocation(irp);

    size = irp_sp->Parameters.DeviceIoControl.OutputBufferLength;
    buf = irp->UserBuffer;

    nt_ret = hid_get_rep_desc(dev_obj, buf, &size);
    if(! NT_SUCCESS(nt_ret)) {
        return nt_ret;
    }

    irp->IoStatus.Information = size;

    return STATUS_SUCCESS;
}

NTSTATUS ioctl_read_rep(IN PDEVICE_OBJECT dev_obj, IN PIRP irp)
{
    PIO_STACK_LOCATION irp_sp;
    ULONG size;
    BYTE *buf;
    NTSTATUS nt_ret;

    irp_sp = IoGetCurrentIrpStackLocation(irp);

    size = irp_sp->Parameters.DeviceIoControl.OutputBufferLength;
    buf = irp->UserBuffer;

    nt_ret = hid_read_rep(dev_obj, buf, &size);
    if(! NT_SUCCESS(nt_ret)) {
        return nt_ret;
    }

    irp->IoStatus.Information = size;

    return STATUS_SUCCESS;
}

NTSTATUS ioctl_write_rep(IN PDEVICE_OBJECT dev_obj, IN PIRP irp)
{
    PHID_XFER_PACKET xfer_packet;
    ULONG size;
    BYTE *buf;
    NTSTATUS nt_ret;

    xfer_packet = (PHID_XFER_PACKET)irp->UserBuffer;
    size = xfer_packet->reportBufferLen;
    buf = xfer_packet->reportBuffer;

    nt_ret = hid_write_rep(dev_obj, buf, &size);
    if(! NT_SUCCESS(nt_ret)) {
        return nt_ret;
    }

    irp->IoStatus.Information = size;

    return STATUS_SUCCESS;
}
